------------------------------------------------------------------
-- Tokeneer ID Station Core Software
--
-- Copyright (2003) United States Government, as represented
-- by the Director, National Security Agency. All rights reserved.
--
-- This material was originally developed by Praxis High Integrity
-- Systems Ltd. under contract to the National Security Agency.
------------------------------------------------------------------

------------------------------------------------------------------
-- Clock
--
-- Description:
--    Provides a "current time" for each system cycle.
--
------------------------------------------------------------------
with BasicTypes,
     Time,
     Time.Interface;

--# inherit BasicTypes,
--#         Time,
--#         Time.Interface;

package Clock
--# own CurrentTime;
is

   ------------------------------------------------------------------
   -- Types
   --
   ------------------------------------------------------------------

   ------------------------------------------------------------------
   -- Poll
   --
   -- Description:
   --    Reads the system clock, and updates the internal CurrentTime.
   --
   -- Traceunit: C.Clock.Poll
   -- Traceto: FD.Interface.PollTime
   ------------------------------------------------------------------

   procedure Poll;
   --# global in     Time.Interface.Now;
   --#           out CurrentTime;
   --# derives CurrentTime from Time.Interface.Now;

   ------------------------------------------------------------------
   -- TheCurrentTime
   --
   -- Description:
   --    Returns CurrentTime recorded at the last poll.
   --
   -- Traceunit: C.Clock.TheCurrentTime
   -- Traceto: FD.RealWorld.State
   ------------------------------------------------------------------

   function TheCurrentTime return Time.T;
   --# global CurrentTime;

--     ------------------------------------------------------------------
--     -- GetNow
--     --
--     -- Description:
--     --    Returns the system time now.
--     --
--     -- Traceunit: C.Clock.GetNow
--     -- Traceto: FD.MonitoredRealWorld.State
--     ------------------------------------------------------------------
--
--     function GetNow return Time.T;
--     --# global Now;

end Clock;







