with Ada.Calendar;
package body Time.Interface
is --# hide Time.Interface;

   function GetNow return T
   is
      T : Ada.Calendar.Time := Ada.Calendar.Clock;

      Year      : Ada.Calendar.Year_Number;
      Month     : Ada.Calendar.Month_Number;
      Day       : Ada.Calendar.Day_Number;
      Sec       : Duration;

      Ret : Time.T;

   begin
      Ada.Calendar.Split
        (Date    => T,
         Year    => Year,
         Month   => Month,
         Day     => Day,
         Seconds => Sec);

--        -- force truncating not rounding to ensure MilliSec doesn't overflow
--        if Sec > 0.0005 then       -- ? che vuol dire ?
--           Sec := Sec - 0.0005;
--        end if;
      Ret := Time.T'(Year => YearsT(Year),
                     Month => MonthsT(Month),
                     Day => DaysT(Day),
                     MilliSec => MilliSecsT(Sec) * MilliSecsInSec);
      return Ret;
   end GetNow;

   function ValidTimeData
     (Year      : in     BasicTypes.Unsigned32T;
      Month     : in     BasicTypes.Unsigned32T;
      Day       : in     BasicTypes.Unsigned32T;
      TimeOfDay : in     DurationT) return Boolean
   is
      TheTime : Ada.Calendar.Time ;
   begin
      TheTime :=
        Ada.Calendar.Time_Of
        (Year    => Ada.Calendar.Year_Number(Year),
         Month   => Ada.Calendar.Month_Number(Month),
         Day     => Ada.Calendar.Day_Number(Day),
         Seconds => Ada.Calendar.Day_Duration( Duration(TimeOfDay) / 1000));

      return True;

   exception when Ada.Calendar.Time_Error =>
         return False;
   end ValidTimeData;

   ------------------------------------------------------------------
   -- ConstructTimeFromDuration
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
   procedure ConstructTimeFromDuration
     (Year      : in     BasicTypes.Unsigned32T;
      Month     : in     BasicTypes.Unsigned32T;
      Day       : in     BasicTypes.Unsigned32T;
      TimeOfDay : in     DurationT;
      TheTime   :    out T;
      Success   :    out Boolean)

   is
   begin
      if Interface.ValidTimeData(Year, Month, Day, TimeOfDay) then
         TheTime := T'(Year => YearsT(Year),
                       Month => MonthsT(Month),
                       Day => DaysT(Day),
                       MilliSec => MilliSecsT(TimeOfDay*MilliSecsInTenthSec));
         Success := True;
      else
         TheTime := ZeroTime;
         Success := False;
      end if;
   end ConstructTimeFromDuration;
   ------------------------------------------------------------------
   -- AddDuration
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
--     function AddDuration
--       (TheTime : T; TheDuration : DurationT ) return T
--     is
--     begin
--        return SparkInterface.AddDuration(TheTime, TheDuration);
end Time.Interface;
