package body Time
is

   ------------------------------------------------------------------
   -- Types and constants
   --
   ------------------------------------------------------------------
      MilliSecsInMin : constant := 60 * MilliSecsInSec;
      MilliSecsInHr : constant := 60 * MilliSecsInMin;

   ------------------------------------------------------------------
   -- Private Operations
   ------------------------------------------------------------------

   ------------------------------------------------------------------
   -- SetStringSegment
   --
   -- Definition:
   --    Sets the section of the string S from SStart to SEnd with
   --    the String representation of the Value.
   --    Pads to the left with '0'.
   --    Truncates if Value is too big to fit.
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
   procedure SetStringSegment
     ( S      : in out String;
       Value  : in     Natural;
       SStart : in     Positive;
       SEnd   : in     Positive)
   --# derives S from *,
   --#                Value,
   --#                SEnd,
   --#                SStart;
   --# pre S'Last >= SEnd and S'First <= SStart;
   is
      V : Natural;
   begin

      V := Value;

      for I in reverse Positive range SStart .. SEnd loop
         --# assert I <= SEnd and I >= SStart
         --#        and I <= S'Last and I >= S'First
         --#        and V in Natural ;
         S(I) := Character'Val(Character'Pos('0') + (V mod 10));
         V    := V / 10;
      end loop;

   end SetStringSegment;

      ------------------------------------------------------------------
   -- GreaterThan
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------

   function GreaterThan ( Left, Right : T ) return Boolean
   is
   begin
      return Left.Year > Right.Year
        or (Left.Year = Right.Year
            and Left.Month > Right.Month )
        or (Left.Year = Right.Year
            and Left.Month = Right.Month
            and Left.Day > Right.Day)
        or (Left.Year = Right.Year
            and Left.Month = Right.Month
            and Left.Day = Right.Day
            and Left.MilliSec > Right.MilliSec);
   end GreaterThan;

   ------------------------------------------------------------------
   -- LessThan
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------

   function LessThan ( Left, Right : T ) return Boolean
   is
   begin
      return Left.Year < Right.Year
        or (Left.Year = Right.Year
            and Left.Month < Right.Month )
        or (Left.Year = Right.Year
            and Left.Month = Right.Month
            and Left.Day < Right.Day)
        or (Left.Year= Right.Year
            and Left.Month = Right.Month
            and Left.Day = Right.Day
            and Left.MilliSec < Right.MilliSec);
   end LessThan;

   ------------------------------------------------------------------
   -- GreaterThanOrEqual
   --
   -- Implemention Notes:
   --    None.
   --
   ------------------------------------------------------------------

   function GreaterThanOrEqual ( Left, Right : T ) return Boolean
   is
   begin
       return GreaterThan (Left, Right) or Left = Right;
   end GreaterThanOrEqual;

   ------------------------------------------------------------------
   -- LessThanOrEqual
   --
   -- Implemention Notes:
   --    None.
   --
   ------------------------------------------------------------------

   function LessThanOrEqual ( Left, Right : T ) return Boolean
   is
   begin
       return LessThan (Left, Right) or Left = Right;
   end LessThanOrEqual;



   ------------------------------------------------------------------
   -- SplitTime
   --
   -- Implementation Notes:
   --    This always rounds down.
   --
   ------------------------------------------------------------------
   procedure SplitTime
     (TheTime : in     T;
      Year    :    out YearsT;
      Month   :    out MonthsT;
      Day     :    out DaysT;
      Hour    :    out HoursT;
      Min     :    out MinutesT)
   is

   begin
      Year  := TheTime.Year;
      Month := TheTime.Month;
      Day   := TheTime.Day;
      Hour  := HoursT (TheTime.MilliSec / MilliSecsInHr);
      Min   := MinutesT ((TheTime.MilliSec mod MilliSecsInHr )
                        / MilliSecsInMin);
   end SplitTime;

   ------------------------------------------------------------------
   -- StartOfDay
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
   function StartOfDay (TheTime : T) return T
   is
   begin
      return T'(Year => TheTime.Year,
                    Month => TheTime.Month,
                    Day => TheTime.Day,
                    MilliSec => MilliSecsT'First);
   end StartOfDay;


   ------------------------------------------------------------------
   -- PrintDuration
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
   function PrintDuration (TheDuration : DurationT ) return DurationTextT
   is
      LocalText : DurationTextT := "hh:mm:ss.s";
      FirstHourIndex    : constant TextI := 1;
      LastHourIndex     : constant TextI := 2;

      FirstMinIndex     : constant TextI := 4;
      LastMinIndex      : constant TextI := 5;

      FirstSecIndex     : constant TextI := 7;
      LastSecIndex      : constant TextI := 8;

      TenthSecIndex     : constant TextI := 10;

      TenthSecsInSec     : constant := 10;
      TenthSecsInMin     : constant := 60 * TenthSecsInSec;
      TenthSecsInHr      : constant := 60 * TenthSecsInMin;
   begin

      SetStringSegment(LocalText,
                       Natural(TheDuration / TenthSecsInHr),
                       FirstHourIndex,
                       LastHourIndex);

      SetStringSegment(LocalText,
                       Natural((TheDuration mod TenthSecsInHr)
                               / TenthSecsInMin),
                       FirstMinIndex,
                       LastMinIndex);

      SetStringSegment(LocalText,
                       Natural((TheDuration mod TenthSecsInMin)
                               / TenthSecsInSec),
                       FirstSecIndex,
                       LastSecIndex);

      SetStringSegment(LocalText,
                       Natural(TheDuration mod TenthSecsInSec),
                       TenthSecIndex,
                       TenthSecIndex);

      return LocalText;
   end PrintDuration;

   ------------------------------------------------------------------
   -- PrintTime
   --
   -- Implementation Notes:
   --    None.
   --
   ------------------------------------------------------------------
   function PrintTime (TheTime : T ) return TextT
   is
      LocalText : TextT := "yyyy-mm-dd hh:mm:ss.s";

      FirstYearIndex    : constant TextI := 1;
      LastYearIndex     : constant TextI := 4;

      FirstMonthIndex   : constant TextI := 6;
      LastMonthIndex    : constant TextI := 7;

      FirstDayIndex     : constant TextI := 9;
      LastDayIndex      : constant TextI := 10;

      FirstHourIndex    : constant TextI := 12;
      LastHourIndex     : constant TextI := 13;

      FirstMinIndex     : constant TextI := 15;
      LastMinIndex      : constant TextI := 16;

      FirstSecIndex     : constant TextI := 18;
      LastSecIndex      : constant TextI := 19;

      TenthSecIndex     : constant TextI := 21;

   begin
      --# assert LocalText'First = TextI'First and
      --#        LocalText'Last = TextI'Last;

      SetStringSegment(LocalText,
                       Natural(TheTime.Year),
                       FirstYearIndex,
                       LastYearIndex);

      SetStringSegment(LocalText,
                       Natural(TheTime.Month),
                       FirstMonthIndex,
                       LastMonthIndex);

      SetStringSegment(LocalText,
                       Natural(TheTime.Day),
                       FirstDayIndex,
                       LastDayIndex);

      SetStringSegment(LocalText,
                       Natural(TheTime.MilliSec / MilliSecsInHr),
                       FirstHourIndex,
                       LastHourIndex);

      SetStringSegment(LocalText,
                       Natural((TheTime.MilliSec mod MilliSecsInHr)
                               / MilliSecsInMin),
                       FirstMinIndex,
                       LastMinIndex);

      SetStringSegment(LocalText,
                       Natural((TheTime.MilliSec mod MilliSecsInMin)
                               / MilliSecsInSec),
                       FirstSecIndex,
                       LastSecIndex);

      SetStringSegment(LocalText,
                       Natural((TheTime.MilliSec mod MilliSecsInSec)
                               / MilliSecsInTenthSec),
                       TenthSecIndex,
                       TenthSecIndex);

      return LocalText;
   end PrintTime;

end Time;
